/*

(c) 2017, 2018 TCE <TCE-ITDevelopers@toyota-ce.com>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/

import "es5-shim";

import angular from "angular";
import angularRoute from "angular-route";

angular
    .module("kobeAuthorization", [
        "ng",
        "ngRoute"
    ])
    //
    .factory("$authorization", [
        "$authorizationAdapter",
        function (authorizationAdapter) {
            //
            var getClaims = function () {
                if (authorizationAdapter) {
                    if (authorizationAdapter.getClaims) {
                        if (typeof authorizationAdapter.getClaims == "function") {
                            return authorizationAdapter.getClaims();
                        }
                        return authorizationAdapter.getClaims;
                    }
                }
                return {};
            };
            //
            var isAuthenticated = function () {
                if (authorizationAdapter) {
                    if (authorizationAdapter.isAuthenticated) {
                        if (typeof authorizationAdapter.isAuthenticated == "function") {
                            return authorizationAdapter.isAuthenticated();
                        }
                        return authorizationAdapter.isAuthenticated;
                    }
                }
                return false;
            };
            //
            var hasClaim = function (type, value) {
                var claims = getClaims();
                if (typeof claims == "undefined" || claims === null) {
                    return false();
                } else if (typeof claims == "object") {
                    if (Array.isArray(claims)) {
                        return claims.find(item => item[type] == value) != null;
                    } else {
                        var claim = claims[type];
                        if (typeof claim == "undefined" || claim === null) {
                            return false;
                        } else if (Array.isArray(claim)) {
                            return claim.find(item => item == value) != null;
                        } else {
                            return claim == value;
                        }
                    }
                }
                return false;
            };
            //
            var resolveRules = function (rules) {
                if (typeof rules == "undefined") {
                    return true;
                } else if (typeof rules == "string") {
                    return hasClaim("role", rules);
                } else if (typeof rules == "object") {
                    if (Array.isArray(rules)) {
                        var result = false;
                        rules.forEach(rule => {
                            result = result || resolveRules(rule);
                        });
                        return result;
                    } else {
                        var result = false;
                        Object.keys(rules).forEach(rule => {
                            result = result || hasClaim(rule, rules[rule]);
                        });
                        return result;
                    }
                }
                return false;
            };
            //
            return {
                //
                get claims() {
                    return getClaims();
                },
                //
                allow: function (rules) {
                    if (typeof rules == "undefined" || rules == null || rules == "") {
                        return isAuthenticated();
                    }
                    return isAuthenticated() && resolveRules(rules, getClaims());
                },
                //
                deny: function (rules) {
                    if (typeof rules == "undefined" || rules == null || rules == "") {
                        return isAuthenticated();
                    }
                    return isAuthenticated() && resolveRules(rules, getClaims());
                },
                //
                authorize: function (rules) {
                    if (typeof rules == "undefined" || rules == null || rules == "") {
                        return isAuthenticated();
                    } else if (typeof rules == "object") {
                        return isAuthenticated() && resolveRules(rules.allow) && !resolveRules(rules.denied);
                    }
                    return false;
                }
            };
        }
    ]);
//
var directiveFactory = function ($name, $animate, $compile, $authorization) {
    //
    var getBlockNodes = function (nodes) {
        var node = nodes[0];
        var endNode = nodes[nodes.length - 1];
        var blockNodes;
        for (var i = 1; node !== endNode && (node = node.nextSibling); i++) {
            if (blockNodes || nodes[i] !== node) {
                if (!blockNodes) {
                    blockNodes = angular.element(slice.call(nodes, 0, i));
                }
                blockNodes.push(node);
            }
        }
        return blockNodes || nodes;
    };
    //
    return {
        multiElement: true,
        transclude: "element",
        priority: 600,
        terminal: true,
        restrict: "A",
        $$tlb: true,
        //
        link: function ($scope, $element, $attr, ctrl, $transclude) {
            //
            var block, childScope, previousElements;
            //
            var watchExpression = function () {
                var result = false;
                var value = $attr[$name];
                //
                switch ($name) {
                    case "ngAllow":
                        result = $authorization.allow(value);
                        break;
                    case "ngDeny":
                        result = !$authorization.deny(value);
                        break;
                    case "ngAuthorize":
                        result = $authorization.authorize(value);
                        break;
                }
                //
                return result;
            };
            //
            $scope.$watch(watchExpression, function (value) {

                if (value) {
                    if (!childScope) {
                        $transclude(function (clone, newScope) {
                            childScope = newScope;
                            clone[clone.length++] = $compile.$$createComment("end " + $name, $attr[$name]);
                            // Note: We only need the first/last node of the cloned nodes.
                            // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                            // by a directive with templateUrl when its template arrives.
                            block = {
                                clone: clone
                            };
                            $animate.enter(clone, $element.parent(), $element);
                        });
                    }
                } else {
                    if (previousElements) {
                        previousElements.remove();
                        previousElements = null;
                    }
                    if (childScope) {
                        childScope.$destroy();
                        childScope = null;
                    }
                    if (block) {
                        previousElements = getBlockNodes(block.clone);
                        $animate.leave(previousElements).done(function (response) {
                            if (response !== false) previousElements = null;
                        });
                        block = null;
                    }
                }
            });
        }
    };
};
//
angular
    .module("kobeAuthorization")
    //
    .directive("ngAllow", [
        "$animate", "$compile", "$authorization",
        function ($animate, $compile, $authorization) {
            return directiveFactory("ngAllow", $animate, $compile, $authorization);
        }
    ])
    //
    .directive("ngDeny", [
        "$animate", "$compile", "$authorization",
        function ($animate, $compile, $authorization) {
            return directiveFactory("ngDeny", $animate, $compile, $authorization);
        }
    ])
    //
    .directive("ngAuthorize", [
        "$animate", "$compile", "$authorization",
        function ($animate, $compile, $authorization) {
            return directiveFactory("ngAuthorize", $animate, $compile, $authorization);
        }
    ]);
//
export default "kobeAuthorization";
